#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <arpa/inet.h>
#include <time.h>

#include "util.hpp"

int debug;

void do_debugf(int level, const char *format, ...)
{
    va_list args;
    va_start(args, format);
    if(debug >= level) {
        vfprintf(stderr, format, args);
        fflush(stderr);
    }
    va_end(args);
}

const char *
format_address(const unsigned char *address)
{
    static char buf[4][INET6_ADDRSTRLEN];
    static int i = 0;
    i = (i + 1) % 4;
    /*
    if(v4mapped(address))
        inet_ntop(AF_INET, address + 12, buf[i], INET6_ADDRSTRLEN);
    else
    */
    inet_ntop(AF_INET6, address, buf[i], INET6_ADDRSTRLEN);
    return buf[i];
}

const char*
human_readable_unit(double value, const char* unit) {
    static char buf[4][24];
    static int cur_buf = 0;
    static const char MULTIPLIERS[] = {' ', 'k', 'M', 'G', 'T'};
    static const int NB_MULTIPLIERS = 5;

    int multiplier = 0;
    double div_value = value;

    cur_buf = (cur_buf + 1) % 4;
    while(div_value >= 1000 && multiplier < NB_MULTIPLIERS) {
        div_value /= 1000;
        multiplier++;
    }
    snprintf(buf[cur_buf], 24, "%.1lf %c%s",
            div_value, MULTIPLIERS[multiplier], unit);
    return buf[cur_buf];
}

void
timespec_diff(const struct timespec t1, const struct timespec t2,
        struct timespec* result)
{
    result->tv_nsec = t1.tv_nsec - t2.tv_nsec;
    result->tv_sec = t1.tv_sec - t2.tv_sec;
    if(result->tv_nsec < 0) {
        result->tv_nsec += 1000*1000*1000;
        result->tv_sec--;
    }
}

uint32_t timespec_us_diff(const struct timespec t1, const struct timespec t2) {
    long nsec_diff = t1.tv_nsec - t2.tv_nsec;
    time_t sec_diff = t1.tv_sec - t2.tv_sec;
    if(nsec_diff < 0) {
        nsec_diff += 1000*1000*1000;
        sec_diff--;
    }

    if(sec_diff > MAX_USEC_DIFF)
        sec_diff = MAX_USEC_DIFF;
    return (nsec_diff / 1000) + (sec_diff * 1000*1000);
}

uint32_t timespec_us_ellapsed(const struct timespec ref) {
    struct timespec now;
    clock_gettime(CLOCK_MONOTONIC, &now);
    return timespec_us_diff(now, ref);
}

uint32_t current_us_timestamp() {
    struct timespec now;
    clock_gettime(CLOCK_MONOTONIC, &now);
    return (now.tv_sec * 1000*1000) + (now.tv_nsec / 1000);
}
